


def is_rtl_language (language:str):
    all_RTLs = [
        "arabic", "hebrew", "persian (farsi)", "pashto", "sindhi", "dhivehi",
        "kurdish (kurmanji)", "uyghur", "yiddish", "ar", "he", "fa", "ps", "sd", "dv",
        "ku", "ug", "yi"
    ]

    if str(language) in all_RTLs:
        return True
    
    return False


from enum import Enum


class GoogleTranslateLanguage (Enum):
    """These are the lanuages that supported with google translate"""
    auto = "auto"
    afrikaans = "afrikaans"
    albanian = "albanian"
    amharic = "amharic"
    arabic = "arabic"
    armenian = "armenian"
    assamese = "assamese"
    aymara = "aymara"
    azerbaijani = "azerbaijani"
    bambara = "bambara"
    basque = "basque"
    belarusian = "belarusian"
    bengali = "bengali"
    bhojpuri = "bhojpuri"
    bosnian = "bosnian"
    bulgarian = "bulgarian"
    catalan = "catalan"
    cebuano = "cebuano"
    chichewa = "chichewa"
    chinese_simplified = "chinese (simplified)"
    chinese_traditional = "chinese (traditional)"
    corsican = "corsican"
    croatian = "croatian"
    czech = "czech"
    danish = "danish"
    dhivehi = "dhivehi"
    dogri = "dogri"
    dutch = "dutch"
    english = "english"
    esperanto = "esperanto"
    estonian = "estonian"
    ewe = "ewe"
    filipino = "filipino"
    finnish = "finnish"
    french = "french"
    frisian = "frisian"
    galician = "galician"
    georgian = "georgian"
    german = "german"
    greek = "greek"
    guarani = "guarani"
    gujarati = "gujarati"
    haitian_creole = "haitian creole"
    hausa = "hausa"
    hawaiian = "hawaiian"
    hebrew = "hebrew"
    hindi = "hindi"
    hmong = "hmong"
    hungarian = "hungarian"
    icelandic = "icelandic"
    igbo = "igbo"
    ilocano = "ilocano"
    indonesian = "indonesian"
    irish = "irish"
    italian = "italian"
    japanese = "japanese"
    javanese = "javanese"
    kannada = "kannada"
    kazakh = "kazakh"
    khmer = "khmer"
    kinyarwanda = "kinyarwanda"
    konkani = "konkani"
    korean = "korean"
    krio = "krio"
    kurdish_kurmanji = "kurdish (kurmanji)"
    kurdish_sorani = "kurdish (sorani)"
    kyrgyz = "kyrgyz"
    lao = "lao"
    latin = "latin"
    latvian = "latvian"
    lingala = "lingala"
    lithuanian = "lithuanian"
    luganda = "luganda"
    luxembourgish = "luxembourgish"
    macedonian = "macedonian"
    maithili = "maithili"
    malagasy = "malagasy"
    malay = "malay"
    malayalam = "malayalam"
    maltese = "maltese"
    maori = "maori"
    marathi = "marathi"
    meiteilon_manipuri = "meiteilon (manipuri)"
    mizo = "mizo"
    mongolian = "mongolian"
    myanmar = "myanmar"
    nepali = "nepali"
    norwegian = "norwegian"
    odia_oriya = "odia (oriya)"
    oromo = "oromo"
    pashto = "pashto"
    persian = "persian"
    polish = "polish"
    portuguese = "portuguese"
    punjabi = "punjabi"
    quechua = "quechua"
    romanian = "romanian"
    russian = "russian"
    samoan = "samoan"
    sanskrit = "sanskrit"
    scots_gaelic = "scots gaelic"
    sepedi = "sepedi"
    serbian = "serbian"
    sesotho = "sesotho"
    shona = "shona"
    sindhi = "sindhi"
    sinhala = "sinhala"
    slovak = "slovak"
    slovenian = "slovenian"
    somali = "somali"
    spanish = "spanish"
    sundanese = "sundanese"
    swahili = "swahili"
    swedish = "swedish"
    tajik = "tajik"
    tamil = "tamil"
    tatar = "tatar"
    telugu = "telugu"
    thai = "thai"
    tigrinya = "tigrinya"
    tsonga = "tsonga"
    turkish = "turkish"
    turkmen = "turkmen"
    twi = "twi"
    ukrainian = "ukrainian"
    urdu = "urdu"
    uyghur = "uyghur"
    uzbek = "uzbek"
    vietnamese = "vietnamese"
    welsh = "welsh"
    xhosa = "xhosa"
    yiddish = "yiddish"
    yoruba = "yoruba"
    zulu = "zulu"


from easynmt import EasyNMT



def translate_using_opusMT (Translatemodel:EasyNMT, src:str, from_language, into_language):

    if from_language == "auto":
        r = Translatemodel.translate(src, target_lang=into_language, 
                                    perform_sentence_splitting=True)
    else:
        r = Translatemodel.translate(src, target_lang=into_language, source_lang=from_language, 
                                 perform_sentence_splitting=True)
    
    return str(r)


if __name__ == "__main__":
    model = EasyNMT('opus-mt')


from ..utils.allowed_props_to_translate import allowed_props_to_translate
from ..tools.translate_using_google import translate_using_google
from ..tools.translate_using_opusMT import translate_using_opusMT
import flet, threading


def translate_control_content (TranslateFletPage_class, control:flet.Control, use_internet:bool=True):
    """
    This function translate the control content.

    It create an attr (last_translated_texts:dict) in the control that stores the last translated
    texts so it reduces unnecessary translation requests.
    """
    
    # Check if the control is skiped so not translate it.
    if control in TranslateFletPage_class.skiped_controls: return

    # start translating the control
    for p in allowed_props_to_translate:
        if hasattr(control, str(p)):
            value = getattr(control, str(p))
            if use_internet and value != None:
                r = translate_using_google(
                    src=value, 
                    from_language=TranslateFletPage_class.from_language.value,
                    into_language=TranslateFletPage_class.into_language.value
                )
                setattr(control, str(p), str(r))
            
            elif use_internet == False and value != None:
                r = translate_using_opusMT(
                    Translatemodel=TranslateFletPage_class.opusMT_model,
                    src=value,
                    from_language=TranslateFletPage_class.from_language.value,
                    into_language=TranslateFletPage_class.into_language.value
                )
                setattr(control, str(p), str(r))



    sub_controls_names = ["controls", "tabs"]
    for i in sub_controls_names:
        if hasattr(control, i):
            for i in getattr(control, str(i)):
                threading.Thread(target=translate_control_content, kwargs={
                    "TranslateFletPage_class" : TranslateFletPage_class,
                    "control" : i,
                    "use_internet" : use_internet
                }, daemon=True).start()
                # translate_control_content(TranslateFletPage_class, i, use_internet)
    

    if hasattr(control, "content"):
        threading.Thread(target=translate_control_content, kwargs={
                    "TranslateFletPage_class" : TranslateFletPage_class,
                    "control" : getattr(control, "content"),
                    "use_internet" : use_internet
                }, daemon=True).start()

    if control.page != None:
        control.update()


from ..utils.google_supported_langaues import GoogleTranslateLanguage
from ..utils.is_rtl import is_rtl_language
from ..tools.translate_control_content import translate_control_content
from easynmt import EasyNMT
import flet, threading, os


class TranslateFletPage:
    """
    This class will translate the whole page into a specific language. 
    The childs controls, page info (title, appbar, ..), etc..

    if `use_internet` is True (Which is not recomended for some cases), That mean
    the translate model is Google translate API, which may give you block band for spam requests.

    if `use_internet` is False, Thats mean the translate model is `opus-mt`, which is a
    local machine learning that trained for many popular languages. It works localy on the device,
    Its use the device performance. So it can lead to a heavy usage.
    """
    def __init__(self,
                page : flet.Page,
                use_internet = True,
                from_language = GoogleTranslateLanguage.auto, 
                into_language = GoogleTranslateLanguage.auto,
                skiped_controls = None
                ) -> None:
        
        self.page = page
        self.from_language = from_language
        self.into_language = into_language

        if isinstance(skiped_controls, list):
            self.skiped_controls = skiped_controls
        else:
            self.skiped_controls = []

        self.__use_internet = use_internet

        if use_internet == False:
            # store ML models
            self.opusMT_model = EasyNMT('opus-mt')

            # Activate the model to be faster in the second request.
            self.opusMT_model.translate(
                "Hi", 
                target_lang="en", 
                source_lang="en"
            )



    def translate_child_controls (self):
        if self.page.appbar != None:
            translate_control_content(self, control=self.page.appbar, use_internet=self.__use_internet)
        for con in self.page.controls:
            threading.Thread(
                target=translate_control_content,
                args=[self],
                kwargs={
                    "control" : con,
                    "use_internet" : self.__use_internet
                }
            ).start()
    
    def activate_local_ML_translation (self):
        """This will activate the local machine learning model to be used in future translation requests"""
        self.__use_internet = False
        # store ML models
        self.opusMT_model = EasyNMT('opus-mt')

        # Activate the model to be faster in the second request.
        self.opusMT_model.translate(
            "Hi", 
            target_lang="en", 
            source_lang="en"
        )
    
    
    def activate_google_translation (self):
        """This will activate the google translation mode, so any future translation requests will be via google API"""
        self.__use_internet = True


    def update(self):
        """This will be called per page.update so it follow the page updates"""
        if is_rtl_language(language=str(self.into_language.value)):
            self.page.rtl = True
        else:
            self.page.rtl = False
        self.page.update()

        self.translate_child_controls()
        self.page.update()


from enum import Enum


class OpusmtLanguage (Enum):
    """
    This class contain all languages supported with OpusMT ML translate.

    Its better with Opusmt to not use `auto`
    """
    auto = "auto"
    af = "af"
    ar = "ar"
    az = "az"
    bn = "bn"
    cs = "cs"
    de = "de"
    en = "en"
    es = "es"
    et = "et"
    fa = "fa"
    fi = "fi"
    fr = "fr"
    gl = "gl"
    gu = "gu"
    he = "he"
    hi = "hi"
    hr = "hr"
    id_language = "id"
    it_language = "it"
    ja = "ja"
    ka = "ka"
    kk = "kk"
    km = "km"
    ko = "ko"
    lt = "lt"
    lv = "lv"
    ml = "ml"
    mn = "mn"
    my = "my"
    ne = "ne"
    nl = "nl"
    pl = "pl"
    ps = "ps"
    pt = "pt"
    ro = "ro"
    ru = "ru"
    si = "si"
    sl = "sl"
    sv = "sv"
    sw = "sw"
    ta = "ta"
    th = "th"
    tl = "tl"
    tr = "tr"
    uk = "uk"
    ur = "ur"
    vi = "vi"
    xh = "xh"
    zh = "zh"


from deep_translator import GoogleTranslator



def translate_using_google (src:str, from_language:str, into_language:str):
    sents = []
    
    current_sent = ""
    num = 0
    for i in src:
        if num >= 250:
            sents.append(current_sent)
            current_sent = ""
            num = 0
        current_sent = current_sent + i
        num = num + 1
    
    sents.append(current_sent)
    res = GoogleTranslator(source=from_language, target=into_language).translate_batch(sents)

    full_res = ""
    for i in res:
        full_res = full_res + i

    return str(full_res)


from setuptools import setup, find_packages

with open("README.md", encoding="utf-8") as f:
    long_des = str(f.read())

setup(
    name='flet_translator',
    version='1.0',
    author='SKbarbon',
    description='A package that help flet developers to make their apps support multiple languages',
    long_description=long_des,
    long_description_content_type='text/markdown',
    url='https://github.com/SKbarbon/flet_translator',
    install_requires=["flet", "EasyNMT", "sacremoses", "deep-translator"],
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows"
    ],
)